/**************************************************************************
*  Notes - a Case Study for the Java AWT Seminar                          *
*  written by Dave Mitchell (zeno@vnet.ibm.com)                           *
*  Java Technical Office - IBM Hursley                                    *
*  January 1996                                                           *
***************************************************************************
*                                                                         *
*  This application is designed to demonstrate/illustrate the main        *
*  features of the AWT covered in the Java AWT Seminar (the second of a   *
*  series to be given by the Hursley Java Group).                         *
*                                                                         *
*  The application is a very simple notepad editor. It allows text files  *
*  to be created or updated. It demonstrates the use of simple menus and  *
*  dialogs.                                                               *
*                                                                         *
***************************************************************************
*                                                                         *
*  Classes and methods used:                                              *
*  java.Lang.String                                                       *
*  java.awt.FileDialog          setDirectory, setFile, getFile, show      *
*  java.awt.MenuBar                                                       *
*  java.awt.TextArea            insertText, getText, setEditable          *
*  java.awt.Frame               setFont, setBackground, setMenuBar        *
*                               resize, pack, add, preferredSize()        *
*  java.awt.Component           hide, show                                *
*  java.awt.Font                                                          *
*  java.awt.MenuItem                                                      *
*  java.awt.Menu                add, addSeparator                         *
*  java.awt.Dialog              add, dispose                              *
*  java.awt.Event                                                         *
*  java.io.BufferedInputStream                                            *
*  java.io.DataInputStream      readLine, close                           *
*  java.io.FileInputStream                                                *
*  java.io.BufferedOutputStream                                           *
*  java.io.DataOutputStream     flush, close, writeBytes                  *
*  java.io.FileOutputStream                                               *
*                                                                         *
***************************************************************************
*  20 Jan 1996 - DM initial version                                       *
*  27 Mar 1996 - DM add shortcut stuff                                    *
*   8 Nov 1996 - DM various minor improvements                            *

**************************************************************************/

import java.awt.*;      // we use quite few AWT classes
import freaklamarsch.awt.*;      // we use quite few AWT classes
import java.io.*;       // and various InputStream and OutputStream classes

public class Notes extends Frame
{
    private TextArea t;                  // this contains the text
    private FLMFileDialog openDialog;       // these two dialogs are
    private FileDialog saveDialog;       // used when loading and saving
    private String filename;             // the name of the file to load/save
    private MenuBar mbar;                // the menu bar
    private Dimension screenSize;

    /*static String queryMenuShortcut()
    {
        String s = System.getProperty("os.name","?");
        if (s.startsWith("Windows 95"))
            return ("&");
        else if (s.startsWith("OS/2"))
            return ("~");
        else
            return ("");
    }*/

    /*private final String shortcut = queryMenuShortcut();
    private final String mnuFile   = shortcut + "File";
    private final String mnuNew    = shortcut + "New";
    private final String mnuOpen   = shortcut + "Open";
    private final String mnuSave   = shortcut + "Save";
    private final String mnuSaveAs = "Save " + shortcut + "As";
    private final String mnuQuit   = shortcut + "Quit";
    private final String mnuHelp   = shortcut + "Help";
    //private final String mnuTest   = shortcut + "Test!";
    private final String mnuAbout  = "A" + shortcut + "bout";*/
    
    private final String mnuFile   = "File";
    private final String mnuNew    = "New";
    private final String mnuOpen   = "Open";
    private final String mnuSave   = "Save";
    private final String mnuSaveAs = "Save As";
    private final String mnuQuit   = "Quit";
    private final String mnuHelp   = "Help";
    private final String mnuAbout  = "About";

    // constructor
    public Notes()
    {
        // make a Frame and give it a title
        super("Java NotePad Editor");

        // get the size of the Screen
        screenSize = Toolkit.getDefaultToolkit().getScreenSize();

        // pick a common font
        Font f = new Font("TimesRoman", Font.PLAIN, 12);
        setFont(f);                       // give the Frame a font
        setBackground(Color.white);       // and a background colour
        filename = "";                    // as yet no filename

        // do some of the work in sub methods (reduces size of the constructor)
        mbar = new MenuBar();
        InitializeMenus(mbar);            // create the menus
        setMenuBar(mbar);                 // give the Frame a menu bar

        InitializeTextArea();             // set up the text area

        // create the two (invisible dialogs)
        openDialog = new FLMFileDialog(this, System.getProperty("user.dir"));
        saveDialog = new FileDialog(this, "Save File...", FileDialog.SAVE);

        this.setSize(screenSize.width, screenSize.height-100);          // makes ourselves a reasonable size
        //pack();                           // fill the client area with the text
        show();                           // make ourselves visible
    }

    // Reads from a file, throws an IOException if an error occured
    public void read(String s) throws IOException
    {
        String line;
        int i =0;
        FileInputStream in = null;
        BufferedInputStream bis = null;
        DataInputStream dataIn = null;

        // opening the file may fail
        try {
            in = new FileInputStream(s);
            bis = new BufferedInputStream(in);
            dataIn = new DataInputStream(bis);
        } catch(IOException e)
        {
            System.out.println("Error in opening file " + s);
            throw e;
        }

        t.hide(); // avoid flashing as the file is loaded
        // read the file a line at a time
        try {
            while ((line = dataIn.readLine()) != null)
            {
                // add the line, with a Newline appended, at the end
                t.insertText(line + "\n", i);
                i += line.length() + 1;     // compute new end
            }
            in.close();                    // we're done with the file
        } catch(IOException e)
        {
            System.out.println("Error in reading file");
            throw e;
        }
        t.show(); // make the text visible
    }

    // Write to a file, throws an IOException if an error occured
    public void write(String s) throws IOException
    {
        FileOutputStream out = null;
        BufferedOutputStream bos = null;
        DataOutputStream dataOut = null;

        // opening the file may fail
        try {
            out = new FileOutputStream(s);
            bos = new BufferedOutputStream(out);
            dataOut = new DataOutputStream(bos);
        } catch(IOException e)
        {
            System.out.println("Error in opening file " + s);
            throw e;
        }

        // we get the text, all in one piece, and write it
        try {
            dataOut.writeBytes(t.getText());
            dataOut.flush();       // should not be necessary, but without
            // it, the last chunk is NOT written
            out.close();
        } catch(IOException e)
        {
            System.out.println("Error in writing to file");
            throw e;
        }
    }

    // Creates the menus and attaches them to the MenuBar
    private void InitializeMenus(MenuBar mbar)
    {

        // first pulldown
        Menu m = new Menu(mnuFile);
        m.add(new MenuItem(mnuNew));
        m.add(new MenuItem(mnuOpen));
        m.add(new MenuItem(mnuSave));
        m.add(new MenuItem(mnuSaveAs));
        m.addSeparator();
        m.add(new MenuItem(mnuQuit));
        mbar.add(m);

        // second pulldown
        m = new Menu(mnuHelp);
        //m.add(new CheckboxMenuItem(mnuTest));
        //m.addSeparator();
        m.add(new MenuItem(mnuAbout));
        mbar.add(m);
    }

    // Creates the TextArea
    private void InitializeTextArea()
    {
        t = new TextArea("", 12, 80);     // 12 lines of 80 columns
        t.setEditable(true);              // not read-only
        add("Center", t);                 // Frame uses BorderLayout
    }

    // Handle events that have occurred
    public boolean handleEvent(Event evt)
    {
        switch(evt.id)
        {
        case Event.WINDOW_DESTROY:     // time to go
            if (evt.target instanceof Frame)
            {
                System.exit(0);
            }
            return true;
        default:
            // all other events get default handling
            return super.handleEvent(evt);
        }
    }

    // default event handler invokes this for event such as menu selections
    public boolean action(Event evt, Object o)
    {
        String s;
        s = evt.arg.toString();
        if (s.equals(mnuOpen))
        {
            openDialog.show();
            try
            {  // getFile() can throw an exception
                filename = openDialog.getDirectory() + openDialog.getFile();
                if (filename != null)
                    read(filename);
            } catch(IOException e)
            {
                System.out.println("File not found");
            }
            return true;
        }
        else if (s.equals(mnuNew))
        {
            t.setText("");                 // clear the text area
        }
        else if (s.equals(mnuSave) || s.equals(mnuSaveAs))
        {
            try
            {
                // if Save As or no filename set, put up the Save dialog
                if (s.equals(mnuSaveAs) || filename == "")
                {
                    saveDialog.setFile("*.*");
                    saveDialog.show();
                    filename = saveDialog.getFile();
                }
                if (filename != null)
                {
                    if (filename.endsWith(".*.*")) // Save Dialog bug
                        filename = filename.substring(0,filename.length()-4);
                    write(filename);
                }
            } catch(IOException e)
            {
                System.out.println("File not found");
            }
            return true;
        }
        /*else if (s.equals(mnuTest))
        {
        if (((CheckboxMenuItem)(evt.target)).getState())
        {
        // should do something!
        resize(size());
        }
        else
        { // do something else!
        t.setBackground(Color.gray);
        resize(size());
        }
        }*/
        else if (s.equals(mnuQuit))
        {
            System.exit(0);                // time to go
        }
        else if (s.equals(mnuAbout))
        {
            // show the About dialog
            AboutDialog ab = new AboutDialog(this);
            ab.show();
        }
        return true;
    }
    public static void main(String args[])
    {
        new Notes();                      // make a Notes editor
    }

    // This subclass of Dialog is NOT public because only one public
    // class can reside in a file and Notes is that class
    class AboutDialog extends Dialog
    {
        public AboutDialog(Notes parent)
        {
            // try to create a modal dialog
            super(parent, "About Java Notepad", true);
            Button b;
            b = new Button("OK");
            add("North",new Label("Simple Java Notepad Editor"));
            add("Center",new Label("written by Dave Mitchell"));
            add("South", b);
            pack();                           // make it just fit
            resize(preferredSize());
            move((screenSize.width-getSize().width)/2,(screenSize.height-getSize().height)/2);
        }

        public boolean handleEvent(Event evt)
        {
            switch(evt.id)
            {
            case Event.WINDOW_DESTROY:     // time to go
                dispose();
                return true;
            case Event.ACTION_EVENT:
                if ("OK".equals(evt.arg))
                { // OK button was pressed
                    dispose();               // kill the dialog
                    return true;
                }
            default:
                return false;
            }
        }
    }

}


