package freaklamarsch.lang;

import freaklamarsch.util.*;
import java.io.*;
import java.awt.*;
import freaklamarsch.awt.*;
import java.awt.event.*;
/**
* This implementation extends the <code>ClassLoader</code> so that given
* a path to a classfile it tries to load this class into the VM.
* It also has methods to ask the user for a <code>File</code> if it can't be found.
* It's a work in progress.
* @author FReAK La Marsch
*/
public class FLMClassLoader extends ClassLoader
{
    /**
    * This is a mapping of all String => Class done by this classloader.  
    * Each Class in the table will either have a null class loader, or 
    * "this" as its classloader
    */
    private FLMHashMap classes = new FLMHashMap();
    private String returnValue = null;
    private String classpath = "";
    
    Frame parent;
    /**
    * The constructor
    */
    public FLMClassLoader()
    {
        this(null, null);
    }

    /**
    * Another constructor
    */
    public FLMClassLoader(String classpath)
    {
        this(null, classpath);
    }


    /**
    * Another constructor
    */
    public FLMClassLoader(Frame parent)
    {
        this(parent, null);
    }

    /**
    * Another constructor
    */
    public FLMClassLoader(Frame parent, String classpath)
    {
        super();
        this.parent = parent;
        this.classpath = classpath;
    }

    /**
    * Calls loadClass(className, true)
    * @author FReAK La Marsch
    * @param     className the name of the class
    * @return	 the resulting <code>Class</code> object
    * @exception ClassNotFoundException if the class could not be found
    */
    public Class loadClass(String file) throws ClassNotFoundException
    {
        return (loadClass(file, true));
    }

    /**
    * This version of loadClass method is called
    * both from loadClass above and from the internal function FindClassFromClass.
    * It checks, if the class is already loaded and if not it will load the class
    * 
    * @author FReAK La Marsch
    * @param     className the name of the class
    * @param     resolve if <code>true</code> then resolve the class
    * @return	 the resulting <code>Class</code> object
    * @exception ClassNotFoundException if the class could not be found
    */
    protected synchronized Class loadClass(String file, boolean resolve) throws ClassNotFoundException, NoClassDefFoundError
    {
        Class loadedClass;
        String name = calculateClassName(file);
        loadedClass = getLoadedClass(name);

        if (loadedClass == null)
        {
            try
            {
                loadedClass = findSystemClass(name);
            }
            catch (ClassNotFoundException cnfe){}
        }

        if (loadedClass == null && file.toUpperCase().endsWith(".CLASS"))
        {
            try
            {
                loadedClass = loadCompiledClass(file);
            }
            catch(Error e){}
        }

        if(loadedClass == null)
        {
            //now we're searching the classpath for that class
            String[] cpSplits = FLMStringUtil.split(classpath, ";");
            for(int i=0; i<cpSplits.length && loadedClass == null; i++)
            {
                try
                {
                    if (!cpSplits[i].endsWith(File.separator))
                    {
                        cpSplits[i] += File.separator;
                    }
                    String pseudoName = name.replace('.', File.separatorChar);
                    loadedClass = loadCompiledClass(cpSplits[i]+pseudoName+".class");
                }
                catch(Exception e)
                {
                }
            }
        }
        if (loadedClass == null)
        {
            if (parent != null)
            {
                file = getFileFromUser(name);
            }
            if(file != null && file.toUpperCase().endsWith((name.replace('.', File.separatorChar)+".class").toUpperCase()))
            {
               loadedClass = loadCompiledClass(file, name);
            }
            else
            {
                throw new ClassNotFoundException();
            }
        }

        if(loadedClass == null)
        {//we've tried everything possible, sorry
            throw new ClassNotFoundException();
        }

        if (resolve)
        {
            resolveClass(loadedClass);
        }
        return loadedClass;
    }

    /*
    * Opens a <code>Dialog</code> and asks the user for the location of a specific class.
    * @return location String of the requestet class
    */
    private String getFileFromUser(String name)
    {
        FLMOkCancelDialog ocd = new FLMOkCancelDialog(parent, "Missing class ...", "The class named\n"+name+"\ncouldn't be found.\nDo you want to browse\nfor that file or\ncancel the process?", "Browse", "Cancel", null);
        ocd.addOkButtonListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent evt)
            {
                FLMFileDialog fd = new FLMFileDialog(parent, System.getProperty("user.dir"), "Select missing class file", new String[]{"*.class"});
                //fd.setFilenameFilter(new FLMFilenameFilter("*.class"));
                fd.setVisible(true);
                returnValue = ""+fd.getDirectory() + fd.getFile();
                fd.dispose();
            }
        });
        ocd.show();
        String returnValueBuffer = returnValue;
        returnValue = null;
        return returnValueBuffer;
    }

    /**
    * checks if a class has already been loaded by this classLoader
    * @param name Name of the class that should be gotten
    */
    private Class getLoadedClass(String name)
    { 
        Class c = findLoadedClass(name);
        if (c==null)
        {
            c = (Class)classes.get(name);
        }
        return c;
    }


    /**
    * loadCompiledClass will load classes into the VM from a known location
    * it reads the given file and converts it to a class
    * 
    * @author FReAK La Marsch
    * @param classPath the complete path of the .class-file in the filesystem
    * @return the resulting <code>Class</code> object 
    * @error ClassFormatError if the given file is not a .class-file
    */
    public synchronized Class loadCompiledClass(String file)
    {
        return loadCompiledClass(file, calculateClassName(file));
    }

    public synchronized Class loadCompiledClass(String file, String className) throws ClassFormatError, NoClassDefFoundError
    {
        Class loadedClass = null;
        File classToRun = new File(file);
        int fileLength = (int) classToRun.length();
        byte[] classBytes = new  byte[fileLength];
        try
        {
	        FileInputStream fis = new FileInputStream(classToRun);
            fis.read(classBytes);
            fis.close();
        }
        catch(IOException ioe)
        {
            return null;
        }
        try
        {
            loadedClass = defineClass(className, classBytes, 0, classBytes.length);
        }
        catch(ClassFormatError cfe)
        {
            throw cfe;
        }
        catch(NoClassDefFoundError ncdfe)
        {
            throw ncdfe;
        }
        if (loadedClass == null)
        {
            throw new NoClassDefFoundError(className);
        }
        classes.put(loadedClass.getName(), loadedClass);
        return loadedClass;
    }
    
    /**
    * calculates the name of a given class file
    * @param classLocation complete path to the .class-File (like c:\mydir\myclass.class)
    * @param classpath classpath ehich is used to load this class
    * @author FReAK La Marsch
    */
    private String calculateClassName(String classLocation)
    {
            return calculateClassName(classLocation, classpath);
    }
    
    /**
    * calculates the name of a given class file
    * @param classLocation complete path to the .class-File (like c:\mydir\myclass.class)
    * @param cp classpath ehich is used to load this class
    * @author FReAK La Marsch
    */
    public static String calculateClassName(String classLocation, String cp)
    {
        String className = classLocation;
        if (className.indexOf(File.separator) != -1)    
        {
            String[] cpSplits = FLMStringUtil.split(cp, ";");
            for (int i=0; i<cpSplits.length; i++)
            {
                if (classLocation.startsWith(cpSplits[i]))
                {
                    className = classLocation.substring(cpSplits[i].length());
                    if (className.startsWith(File.separator))
                    {
                        className = className.substring(1);
                    }
                    className = className.substring(0, className.length()-6).replace(File.separatorChar, '.');
                    break;
                }
            }
        }
        if (className.equals(classLocation) && classLocation.toUpperCase().endsWith(".CLASS"))
        {
            int lastSep = (classLocation.lastIndexOf(File.separator)!= -1) ? classLocation.lastIndexOf(File.separator) : 0;
            className = classLocation.substring(lastSep+1, classLocation.length()-6);
        }
        return className;
    }
}
